(function() {
  var ThermosContext, config, doctypes, elements, normalizeUrl, options, tags, type, typeOf;
  var __slice = Array.prototype.slice, __hasProp = Object.prototype.hasOwnProperty, __indexOf = Array.prototype.indexOf || function(item) { for (var i = 0, l = this.length; i < l; i++) { if (__hasProp.call(this, i) && this[i] === item) return i; } return -1; };

  this.version = '0.0.1';

  typeOf = function(obj) {
    return Object.prototype.toString.call(obj).slice(8, -1);
  };

  config = {
    ROOT_JS_URL: '/javascripts/',
    ROOT_CSS_URL: "/stylesheets/"
  };

  this.configure = function(opts) {
    return config.helpers = opts.helpers;
  };

  this.resetConfig = function(opts) {
    return config = {};
  };

  this.options = options = {
    DEFAULT_DOCTYPE: 5
  };

  elements = {
    regular: 'a abbr address article aside audio b bdi bdo blockquote body button\
 canvas caption cite code colgroup datalist dd del details dfn div dl dt em\
 fieldset figcaption figure footer form h1 h2 h3 h4 h5 h6 head header hgroup\
 html i iframe ins kbd label legend li map mark menu meter nav noscript object\
 ol optgroup option output p pre progress q rp rt ruby s samp script section\
 select small span strong style sub summary sup table tbody td textarea tfoot\
 th thead time title tr u ul video',
    "void": 'area base br col command embed hr img input keygen link meta param\
 source track wbr',
    obsolete: 'applet acronym bgsound dir frameset noframes isindex listing\
 nextid noembed plaintext rb strike xmp big blink center font marquee multicol\
 nobr spacer tt',
    obsolete_void: 'basefont frame'
  };

  for (type in elements) {
    tags = elements[type];
    elements[type] = tags.split(/\s+/);
  }

  doctypes = {
    '5': '<!DOCTYPE html>',
    'xml': '<?xml version="1.0" encoding="utf-8" ?>',
    'transitional': '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">',
    'strict': '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">',
    'frameset': '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Frameset//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-frameset.dtd">',
    '1.1': '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.1//EN" "http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd">',
    'basic': '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML Basic 1.1//EN" "http://www.w3.org/TR/xhtml-basic/xhtml-basic11.dtd">',
    'mobile': '<!DOCTYPE html PUBLIC "-//WAPFORUM//DTD XHTML Mobile 1.2//EN" "http://www.openmobilealliance.org/tech/DTD/xhtml-mobile12.dtd">',
    'ce': '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "ce-html-1.0-transitional.dtd">'
  };

  normalizeUrl = function(root, url, ext) {
    url = url.substr(-ext.length) !== ext ? url + ext : url;
    if (url[0] === '/' || /^[-a-z]+:\/\//.test(url)) {
      return url;
    } else {
      return root + url;
    }
  };

  ThermosContext = (function() {
    var element, _fn, _i, _len, _ref;
    var _this = this;

    function ThermosContext(opts, template) {
      var func, name, _ref, _ref2;
      if (opts == null) opts = {};
      this.buffer = [];
      this.template = template;
      this.locals = opts.locals || {};
      _ref = config.helpers || {};
      for (name in _ref) {
        func = _ref[name];
        this[name] = func;
      }
      _ref2 = opts.helpers || {};
      for (name in _ref2) {
        func = _ref2[name];
        this[name] = func;
      }
    }

    _ref = elements.regular.concat(elements["void"]);
    _fn = function(element) {
      return ThermosContext.prototype[element] = function() {
        return this.tag.apply(this, [element].concat(__slice.call(arguments)));
      };
    };
    for (_i = 0, _len = _ref.length; _i < _len; _i++) {
      element = _ref[_i];
      _fn(element);
    }

    ThermosContext.prototype.render = function(template, opts) {
      if (opts == null) opts = {};
      this.template.call(this, this.locals);
      return this.buffer.join('');
    };

    ThermosContext.prototype.doctype = function(version) {
      if (version == null) version = options.DEFAULT_DOCTYPE;
      return this.buffer.push(doctypes[String(version)]);
    };

    ThermosContext.prototype.text = function(txt) {
      return this.buffer.push(String(txt));
    };

    ThermosContext.prototype.js = function(url) {
      url = normalizeUrl(config.ROOT_JS_URL, url, '.js');
      return this.script({
        type: "text/javascript",
        src: url
      });
    };

    ThermosContext.prototype.css = function(url) {
      url = normalizeUrl(config.ROOT_CSS_URL, url, '.css');
      return this.link({
        type: "text/css",
        rel: "stylesheet",
        media: "screen",
        href: url
      });
    };

    ThermosContext.prototype.link_to = function(text, url) {
      url = normalizeUrl('', url, '');
      return this.a({
        href: url
      }, text);
    };

    ThermosContext.prototype.tag = function() {
      var arg, args, attrs, classRegex, data, func, groups, i, key, tagName, text, val, value, _len2;
      tagName = arguments[0], args = 2 <= arguments.length ? __slice.call(arguments, 1) : [];
      attrs = {};
      for (i = 0, _len2 = args.length; i < _len2; i++) {
        arg = args[i];
        switch (typeOf(arg)) {
          case 'Function':
            func = arg;
            break;
          case 'String':
            if (i === 0 && /^([#.][a-zA-Z][\w-]*)+$/.test(arg)) {
              classRegex = /\.([^#.]+)/g;
              if (groups = /#([^#.]+)/.exec(arg)) attrs["id"] = groups.pop();
              while (groups = classRegex.exec(arg)) {
                (attrs["class"] || (attrs["class"] = [])).push(groups[1]);
              }
              if ("class" in attrs) attrs["class"] = attrs["class"].join(' ');
            } else {
              text = arg;
            }
            break;
          case 'Object':
            for (key in arg) {
              value = arg[key];
              attrs[key] = value;
            }
        }
      }
      if (data = attrs.data) {
        delete attrs.data;
        for (key in data) {
          val = data[key];
          key = key.replace(/_/g, '-');
          attrs["data-" + key] = val;
        }
      }
      attrs = ((function() {
        var _results;
        _results = [];
        for (key in attrs) {
          value = attrs[key];
          _results.push(" " + key + "=\"" + value + "\"");
        }
        return _results;
      })()).join('');
      if (__indexOf.call(elements["void"], tagName) >= 0) {
        this.text("<" + tagName + attrs + "/>");
      } else {
        this.text("<" + tagName + attrs + ">");
        if (func != null) this.text(func.call(this));
        if (text != null) this.text(text);
        this.text("</" + tagName + ">");
      }
      return "";
    };

    return ThermosContext;

  }).call(this);

  this.render = function(opts, template) {
    var cc;
    if (opts == null) opts = {};
    if (arguments.length === 1) {
      template = opts;
      opts = {};
    }
    cc = new ThermosContext(opts, template);
    return cc.render();
  };

}).call(this);
